@@ -27,14 +27,15 @@ __copyright__ = "Copyright 2020 Brandon M. Pace"
 __license__ = "GNU LGPL 3+"
 __maintainer__ = "Brandon M. Pace"
 __status__ = "Development"
-__version__ = "0.0.1"
+__version__ = "0.0.2"
 
 
 import ipaddress
 import logging
 import re
 
-from typing import Callable, List, Optional, Union
+from collections import defaultdict
+from typing import Callable, Dict, List, Optional, Tuple, Union
 
 
 logger = logging.getLogger(__name__)
@@ -42,7 +43,7 @@ logger = logging.getLogger(__name__)
 dotted_ip_re = r"(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){0,3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)"
 netstat_line_re = rf"^({dotted_ip_re}|default)\s+({dotted_ip_re}|\*)\s+({dotted_ip_re})\s+\w+\s+\d+\s\d+\s+\d+\s([\w\d\-\.\:]+)"
 netstat_line_rec = re.compile(netstat_line_re)
-linux_route_line_re = rf"^({dotted_ip_re}|default)\s+({dotted_ip_re}|\*)\s+({dotted_ip_re})\s+\w+\s+(\d+)\s+\d+\s+\d+\s([\w\d\-\.\:]+)"
+linux_route_line_re = rf"^({dotted_ip_re}|default)\s+({dotted_ip_re}|\*)\s+({dotted_ip_re})\s+\w+\s+(\d+)\s\s+\d+\s+\d+\s([\w\d\-\.\:]+)"
 linux_route_line_rec = re.compile(linux_route_line_re)
 windows_route_print_v4_line_re = rf"^\s+({dotted_ip_re})\s+({dotted_ip_re})\s+({dotted_ip_re}|On-link)\s+({dotted_ip_re})\s+(\d+)"
 windows_route_print_v4_line_rec = re.compile(windows_route_print_v4_line_re)
@@ -170,8 +171,8 @@ class Route:
     def __hash__(self):
         return hash(
             (
-                self._network,
-                self._gateway,
+                self._network.exploded,
+                self._gateway.exploded if self._gateway else "None",
                 self._metric,
                 self._interface
             )
@@ -478,7 +479,7 @@ class RoutingTable:
                     elif match.netmask > final_match.netmask:
                         logger.warning(f"POSSIBLE BUG: the list is likely not sorted properly!")
                         final_match = match
-                    elif (match.network == final_match.network) and (match.netmask == final_match.netmask):
+                    elif match.network.exploded == final_match.network.exploded:
                         if self._prefer_lowest_metric:
                             if match.metric < final_match.metric:
                                 final_match = match
@@ -538,3 +539,60 @@ class RoutingTable:
     @property
     def routes(self):
         return self._routes.copy()
+
+
+input_types: Tuple[str, ...] = ("ip route", "netstat -r", "route (Linux)", "route print (Windows)")
+
+
+input_type_route_function_map: Dict[str, Callable] = {
+    "ip route": Route.from_ip_route_lines,
+    "netstat -r": Route.from_linux_netstat_lines,
+    "route (Linux)": Route.from_linux_route_lines,
+    "route print (Windows)": Route.from_windows_route_print_lines
+}
+assert all(item in input_type_route_function_map for item in input_types)
+
+
+input_type_table_function_map: Dict[str, Callable] = {
+    "ip route": RoutingTable.from_ip_route_lines,
+    "netstat -r": RoutingTable.from_linux_netstat_lines,
+    "route (Linux)": RoutingTable.from_linux_route_lines,
+    "route print (Windows)": RoutingTable.from_windows_route_print_lines
+}
+assert all(item in input_type_table_function_map for item in input_types)
+
+
+input_type_id_function_map: Dict[str, Callable] = {
+    "ip route": is_ip_route_line,
+    "netstat -r": is_netstat_route_line,
+    "route (Linux)": is_linux_route_line,
+    "route print (Windows)": is_route_print_v4_line
+}
+assert all(item in input_type_id_function_map for item in input_types)
+
+
+def function_for_content_type(input_type: str, table: bool = True) -> Callable:
+    """Get the function to create a Route or RoutingTable from a given input type"""
+    if input_type in input_types:
+        if table:
+            return input_type_table_function_map[input_type]
+        else:
+            return input_type_route_function_map[input_type]
+    else:
+        raise ValueError(f"Invalid content type: '{input_type}'")
+
+
+def identify_input(lines: List[str]) -> List[str]:
+    """Identify what supported type(s) of output are contained in lines (if any)"""
+    matches: List[str] = []
+    match_counts = defaultdict(int)
+
+    for line in lines:
+        for match_type, match_func in input_type_id_function_map.items():
+            if match_func(line):
+                match_counts[match_type] += 1
+
+    for match_type in match_counts:
+        matches.append(match_type)
+
+    return matches
